# flake8: noqa
from __future__ import absolute_import, division, print_function

__version__ = '2.0.0'

import utool as ut
ut.noinject(__name__, '[guitool_ibeis.__init__]')


#try:
#    # try seeing if importing plottool before any guitool_ibeis things helps
#    import plottool
#except Exception as ex:
#    import utool as ut
#    ut.printex(ex, 'tried to import plottool to solve win crash')
#    raise
#    #pass

#print('__guitool__1')
from guitool_ibeis import __PYQT__
#print('__guitool__2')

from guitool_ibeis import api_item_model
from guitool_ibeis import api_table_view
from guitool_ibeis import api_tree_view
from guitool_ibeis import api_item_widget
from guitool_ibeis import stripe_proxy_model

from guitool_ibeis import guitool_tables
from guitool_ibeis import guitool_dialogs
from guitool_ibeis import guitool_decorators
from guitool_ibeis import guitool_delegates
from guitool_ibeis import guitool_components
from guitool_ibeis import guitool_main
from guitool_ibeis import guitool_misc
from guitool_ibeis import qtype

import utool

print, rrr, profile = utool.inject2(__name__, '[guitool_ibeis]')

def reload_subs():
    """Reloads utool and submodules """
    rrr()
    if hasattr(guitool_tables, 'rrr'):
        guitool_tables.rrr()
    if hasattr(guitool_dialogs, 'rrr'):
        guitool_dialogs.rrr()
    if hasattr(guitool_decorators, 'rrr'):
        guitool_decorators.rrr()
    if hasattr(guitool_main, 'rrr'):
        guitool_main.rrr()
    if hasattr(guitool_misc, 'rrr'):
        guitool_misc.rrr()
    if hasattr(api_item_model, 'rrr'):
        api_item_model.rrr()
    if hasattr(qtype, 'rrr'):
        qtype.rrr()
    if hasattr(guitool_components, 'rrr'):
        guitool_components.rrr()

rrrr = reload_subs


IMPORT_TUPLES = [
    ('guitool_main', None),
    ('guitool_components', None),
    ('guitool_dialogs', None),
    ('guitool_decorators', None),
    ('guitool_misc', None),
    ('api_item_model', None),
    ('api_tree_view', None),
    ('api_table_view', None),
    ('qtype', None),
    ('stripe_proxy_model', None),
    ('filter_proxy_model', None),
]
"""
python -c "import guitool_ibeis" --dump-guitool_ibeis-init
python -c "import guitool_ibeis" --update-guitool_ibeis-init
"""
__DYNAMIC__ = not ut.get_argflag('--nodyn')
DOELSE = False
if __DYNAMIC__:
    # TODO: import all utool external prereqs. Then the imports will not import
    # anything that has already in a toplevel namespace
    # COMMENTED OUT FOR FROZEN __INIT__
    # Dynamically import listed util libraries and their members.
    from utool._internal import util_importer
    # FIXME: this might actually work with rrrr, but things arent being
    # reimported because they are already in the modules list
    ignore_endswith = ['_cyth']
    ignore_list = ['Qt']
    import_execstr = util_importer.dynamic_import(__name__, IMPORT_TUPLES,
                                                  ignore_endswith=ignore_endswith,
                                                  ignore_list=ignore_list,
                                                  verbose=False)
    exec(import_execstr)
    DOELSE = False
else:
    # Do the nonexec import (can force it to happen no matter what if alwyas set
    # to True)
    DOELSE = True

# This screws up dynamic_import if it is placed before
from guitool_ibeis.guitool_tables import *
from guitool_ibeis.guitool_dialogs import *
from guitool_ibeis.guitool_decorators import *
from guitool_ibeis.guitool_delegates import *
from guitool_ibeis.guitool_components import *
from guitool_ibeis.guitool_main import *
from guitool_ibeis.guitool_misc import *
from guitool_ibeis.api_item_model import *
from guitool_ibeis.api_table_view import *
from guitool_ibeis.api_tree_view import *
from guitool_ibeis.api_item_widget import *
from guitool_ibeis.stripe_proxy_model import *
from guitool_ibeis.filter_proxy_model import *
from guitool_ibeis.qtype import *

if DOELSE:
    pass
    # <AUTOGEN_INIT>

    from guitool_ibeis import guitool_main
    from guitool_ibeis import guitool_components
    from guitool_ibeis import guitool_dialogs
    from guitool_ibeis import guitool_decorators
    from guitool_ibeis import guitool_misc
    from guitool_ibeis import api_item_model
    from guitool_ibeis import api_tree_view
    from guitool_ibeis import api_table_view
    from guitool_ibeis import qtype
    from guitool_ibeis import stripe_proxy_model
    from guitool_ibeis import filter_proxy_model
    from guitool_ibeis.guitool_main import (GUITOOL_PYQT_VERSION, GuitoolApplication,
                                      IS_ROOT_WINDOW, QAPP, QUIET, VERBOSE,
                                      activate_qwindow, ensure_qapp,
                                      ensure_qtapp, exit_application,
                                      get_qtapp, ping_python_interpreter,
                                      qtapp_loop, qtapp_loop_nonblocking,
                                      remove_pyqt_input_hook,)
    from guitool_ibeis.guitool_components import (ALIGN_DICT, BlockSignals,
                                            ConfigConfirmWidget, DEBUG_WIDGET,
                                            GuiProgContext, GuitoolWidget,
                                            PROG_TEXT, ProgHook,
                                            ResizableTextEdit, SimpleTree,
                                            Spoiler, WIDGET_BASE, adjust_font,
                                            fix_child_attr_heirarchy,
                                            fix_child_size_heirarchy,
                                            getAvailableFonts, get_nested_attr,
                                            get_widget_text_width,
                                            layoutSplitter, make_style_sheet,
                                            msg_event, newButton, newCheckBox,
                                            newComboBox, newFont, newFrame,
                                            newLabel, newLineEdit, newMenu,
                                            newMenuAction, newMenubar,
                                            newOutputLog, newProgressBar,
                                            newQPoint, newScrollArea,
                                            newSizePolicy, newSplitter,
                                            newTabWidget, newTextEdit,
                                            newToolbar, newWidget,
                                            print_widget_heirarchy,
                                            prop_text_map, rectify_qt_const,
                                            walk_widget_heirarchy,)
    from guitool_ibeis.guitool_dialogs import (ResizableMessageBox, SELDIR_CACHEID,
                                         are_you_sure, build_nested_qmenu,
                                         connect_context_menu, msgbox,
                                         newDirectoryDialog, newFileDialog,
                                         popup_menu, select_directory,
                                         select_files, select_images,
                                         user_info, user_input, user_option,
                                         user_question,)
    from guitool_ibeis.guitool_decorators import (DEBUG, checks_qt_error, signal_,
                                            slot_,)
    from guitool_ibeis.guitool_misc import (ALT_KEY, BlockContext, GUILoggingHandler,
                                      GUILoggingSender, QLoggedOutput,
                                      find_used_chars, get_cplat_tab_height,
                                      get_view_selection_as_str,
                                      make_option_dict, make_word_hotlinks,)
    from guitool_ibeis.api_item_model import (APIItemModel, API_MODEL_BASE,
                                        ChangeLayoutContext, QVariantHack,
                                        VERBOSE_MODEL,
                                        default_method_decorator,
                                        simple_thumbnail_widget, updater,)
    from guitool_ibeis.api_tree_view import (APITreeView, API_VIEW_BASE,
                                       testdata_tree_view,)
    from guitool_ibeis.api_table_view import (APITableView,)
    from guitool_ibeis.qtype import (ItemDataRoles, LOCALE, QLocale, QString,
                               QT_BUTTON_TYPES, QT_COMBO_TYPES,
                               QT_DELEGATE_TYPES, QT_ICON_TYPES,
                               QT_IMAGE_TYPES, QT_PIXMAP_TYPES, QVariant,
                               SIMPLE_CASTING, cast_from_qt, cast_into_qt,
                               infer_coltype, locale_float, numpy_to_qicon,
                               numpy_to_qpixmap, qindexinfo, to_qcolor,)
    from guitool_ibeis.stripe_proxy_model import (STRIPE_PROXY_BASE,
                                            STRIP_PROXY_META_CLASS,
                                            STRIP_PROXY_SIX_BASE,
                                            StripeProxyModel,)
    from guitool_ibeis.filter_proxy_model import (BASE_CLASS, FilterProxyModel,)
    import utool
    print, rrr, profile = utool.inject2(__name__, '[guitool_ibeis]')


    def reassign_submodule_attributes(verbose=1):
        """
        Updates attributes in the __init__ modules with updated attributes
        in the submodules.
        """
        import sys
        if verbose and '--quiet' not in sys.argv:
            print('dev reimport')
        # Self import
        import guitool_ibeis
        # Implicit reassignment.
        seen_ = set([])
        for tup in IMPORT_TUPLES:
            if len(tup) > 2 and tup[2]:
                continue  # dont import package names
            submodname, fromimports = tup[0:2]
            submod = getattr(guitool_ibeis, submodname)
            for attr in dir(submod):
                if attr.startswith('_'):
                    continue
                if attr in seen_:
                    # This just holds off bad behavior
                    # but it does mimic normal util_import behavior
                    # which is good
                    continue
                seen_.add(attr)
                setattr(guitool_ibeis, attr, getattr(submod, attr))


    def reload_subs(verbose=1):
        """ Reloads guitool_ibeis and submodules """
        if verbose:
            print('Reloading guitool_ibeis submodules')
        rrr(verbose > 1)
        def wrap_fbrrr(mod):
            def fbrrr(*args, **kwargs):
                """ fallback reload """
                if verbose > 0:
                    print('Auto-reload (using rrr) not setup for mod=%r' % (mod,))
            return fbrrr
        def get_rrr(mod):
            if hasattr(mod, 'rrr'):
                return mod.rrr
            else:
                return wrap_fbrrr(mod)
        def get_reload_subs(mod):
            return getattr(mod, 'reload_subs', wrap_fbrrr(mod))
        get_rrr(guitool_main)(verbose > 1)
        get_rrr(guitool_components)(verbose > 1)
        get_rrr(guitool_dialogs)(verbose > 1)
        get_rrr(guitool_decorators)(verbose > 1)
        get_rrr(guitool_misc)(verbose > 1)
        get_rrr(api_item_model)(verbose > 1)
        get_rrr(api_tree_view)(verbose > 1)
        get_rrr(api_table_view)(verbose > 1)
        get_rrr(qtype)(verbose > 1)
        get_rrr(stripe_proxy_model)(verbose > 1)
        get_rrr(filter_proxy_model)(verbose > 1)
        rrr(verbose > 1)
        try:
            # hackish way of propogating up the new reloaded submodule attributes
            reassign_submodule_attributes(verbose=verbose)
        except Exception as ex:
            print(ex)
    rrrr = reload_subs
    # </AUTOGEN_INIT>



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
import sys
import utool as ut


def run_tests():
    # Build module list and run tests
    import sys
    ut.change_term_title('RUN GUITOOL TESTS')
    exclude_doctests_fnames = set([
    ])
    exclude_dirs = [
        '_broken',
        'old',
        'tests',
        'timeits',
        '_scripts',
        '_timeits',
        '_doc',
        'notebook',
    ]
    dpath_list = ['guitool_ibeis']
    doctest_modname_list = ut.find_doctestable_modnames(
        dpath_list, exclude_doctests_fnames, exclude_dirs)

    for modname in doctest_modname_list:
        exec('import ' + modname, globals(), locals())
    module_list = [sys.modules[name] for name in doctest_modname_list]
    nPass, nTotal, failed_cmd_list = ut.doctest_module_list(module_list)
    if nPass != nTotal:
        return 1
    else:
        return 0

if __name__ == '__main__':
    import multiprocessing
    multiprocessing.freeze_support()
    retcode = run_tests()
    sys.exit(retcode)



from __future__ import absolute_import, division, print_function
cimport cython
import cython
#from libc.stdlib cimport malloc, free  # TODO: convert lists to arrays


global CYTHONIZED
CYTHONIZED = True


cdef class TreeNode:
    cdef long id_
    cdef long level
    cdef list child_nodes
    cdef TreeNode parent_node

    def __init__(TreeNode self, long id_, TreeNode parent_node, long level):
        self.id_ = id_
        self.parent_node = parent_node
        self.child_nodes = []
        self.level = level

    def __getitem__(self, long index):
        return self.get_child(index)

    cpdef set_children(TreeNode self, list child_nodes):
        self.child_nodes = child_nodes

    cpdef list get_children(TreeNode self):
        return self.child_nodes

    cpdef TreeNode get_child(self, long index):
        return self.child_nodes[index]

    cpdef get_parent(TreeNode self):
        return self.parent_node

    cpdef long child_index(self, TreeNode child_node):
        return self.child_nodes.index(child_node)

    cpdef long get_num_children(TreeNode self):
        return len(self.child_nodes)

    cpdef long get_id(TreeNode self):
        return self.id_

    cpdef long get_row(TreeNode self):
        cdef list sibling_nodes = self.parent_node.child_nodes
        cdef long row = sibling_nodes.index(self)
        return row

    cpdef long get_level(TreeNode self):
        return self.level


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef TreeNode _populate_tree_recursive(TreeNode parent_node,
                             list child_ids,
                             long num_levels,
                             list ider_list,
                             long level):
    """ Recursively builds the tree structure """
    cdef:
        size_t ix
        long id_
        list child_nodes
        TreeNode next_node
        list next_ids
    if level == num_levels - 1:
        #def TreeNode* child_nodes = malloc(sizeof(TreeNode) * len(child_ids))
        child_nodes = [None] * len(child_ids)
        for ix in range(len(child_ids)):
            id_ = child_ids[ix]
            child_nodes[ix] = TreeNode(id_, parent_node, level)
        #child_nodes = [TreeNode(id_, parent_node, level) for id_ in child_ids]
    else:
        child_nodes = [None] * len(child_ids)
        for ix in range(len(child_ids)):
            id_ = child_ids[ix]
            next_ids = ider_list[level + 1](id_)
            next_node = TreeNode(id_, parent_node, level)
            child_nodes[ix] = _populate_tree_recursive(next_node, next_ids, num_levels, ider_list, level + 1)
        #child_nodes =  [ for id_ in child_ids]
    parent_node.child_nodes = child_nodes
    return parent_node


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef _populate_tree_iterative(TreeNode root_node, long num_levels, list ider_list):
    """ Iteratively builds the tree structure. I dont quite trust this yet """
    cdef: 
        TreeNode parent_node
        size_t level
        size_t ix
        size_t sx
        long id_
        list root_ids
        list parent_node_list
        list ids_list
        list id_list
        list next_ids
        list node_list
        list new_node_lists 
        list new_ids_lists  

    root_ids = ider_list[0]()
    ids_list = [root_ids]
    parent_node_list = [root_node]
    for level in range(num_levels):
        #print('------------ level=%r -----------' % (level,))
        #print(utool.repr2(locals()))
        new_node_lists = []
        new_ids_lists  = []
        for sx in range(len(ids_list)):
            parent_node = parent_node_list[sx]
            id_list = ids_list[sx] 
            node_list = [None] * len(id_list)
            #node_list =  [TreeNode(id_, parent_node, level) for id_ in id_list]
            for ix, id_ in enumerate(id_list):
                node_list[ix] = TreeNode(id_, parent_node, level) 
            if level + 1 < num_levels:
                next_ids =  ider_list[level + 1](id_list)
                #[child_ider(id_) for id_ in child_ids]
            else:
                next_ids = []
            parent_node.child_nodes = node_list
            new_node_lists.extend(node_list)
            new_ids_lists.extend(next_ids)
        parent_node_list = new_node_lists
        ids_list = new_ids_lists


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef TreeNode build_internal_structure(object model):
    #from guitool_ibeis.api_item_model import *
    ider_list = model.iders
    num_levels = len(ider_list)
    USE_RECURSIVE = True
    if USE_RECURSIVE:
        # I trust this code more although it is slightly slower
        if num_levels == 0:
            root_id_list = []
        else:
            root_id_list = ider_list[0]()
        root_node = TreeNode(-1, None, -1)
        level = 0
        #with nogil:
        _populate_tree_recursive(root_node, root_id_list, num_levels, ider_list, level)
    else:
        # TODO: Vet this code a bit more.
        root_node = TreeNode(-1, None, -1)
        #with nogil:
        _populate_tree_iterative(root_node, num_levels, ider_list)
    #print(root_node.full_str())
    #assert root_node.__dict__, "root_node.__dict__ is empty"
    return root_node



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
from os.path import dirname
from os.path import join
from os.path import sys
from os.path import exists


def parse_version(fpath):
    """
    Statically parse the version number from a python file
    """
    import ast
    if not exists(fpath):
        try2 = join(dirname(__file__), fpath)
        if exists(try2):
            fpath = try2
        else:
            raise ValueError('fpath={!r} does not exist'.format(fpath))
    with open(fpath, 'r') as file_:
        sourcecode = file_.read()
    pt = ast.parse(sourcecode)
    class VersionVisitor(ast.NodeVisitor):
        def visit_Assign(self, node):
            for target in node.targets:
                if getattr(target, 'id', None) == '__version__':
                    self.version = node.value.s
    visitor = VersionVisitor()
    visitor.visit(pt)
    return visitor.version


def native_mb_python_tag():
    import sys
    import platform
    major = sys.version_info[0]
    minor = sys.version_info[1]
    ver = '{}{}'.format(major, minor)
    if platform.python_implementation() == 'CPython':
        # TODO: get if cp27m or cp27mu
        impl = 'cp'
        if ver == '27':
            IS_27_BUILT_WITH_UNICODE = True  # how to determine this?
            if IS_27_BUILT_WITH_UNICODE:
                abi = 'mu'
            else:
                abi = 'm'
        else:
            abi = 'm'
    else:
        raise NotImplementedError(impl)
    mb_tag = '{impl}{ver}-{impl}{ver}{abi}'.format(**locals())
    return mb_tag


def parse_description():
    """
    Parse the description in the README file

    CommandLine:
        pandoc --from=markdown --to=rst --output=README.rst README.md
        python -c "import setup; print(setup.parse_description())"
    """
    from os.path import dirname, join, exists
    readme_fpath = join(dirname(__file__), 'README.rst')
    # This breaks on pip install, so check that it exists.
    if exists(readme_fpath):
        with open(readme_fpath, 'r') as f:
            text = f.read()
        return text
    return ''


def parse_requirements(fname='requirements.txt', with_version=False):
    """
    Parse the package dependencies listed in a requirements file but strips
    specific versioning information.

    Args:
        fname (str): path to requirements file
        with_version (bool, default=False): if true include version specs

    Returns:
        List[str]: list of requirements items

    CommandLine:
        python -c "import setup; print(setup.parse_requirements())"
        python -c "import setup; print(chr(10).join(setup.parse_requirements(with_version=True)))"
    """
    from os.path import exists
    import re
    require_fpath = fname

    def parse_line(line):
        """
        Parse information from a line in a requirements text file
        """
        if line.startswith('-r '):
            # Allow specifying requirements in other files
            target = line.split(' ')[1]
            for info in parse_require_file(target):
                yield info
        else:
            info = {'line': line}
            if line.startswith('-e '):
                info['package'] = line.split('#egg=')[1]
            else:
                # Remove versioning from the package
                pat = '(' + '|'.join(['>=', '==', '>']) + ')'
                parts = re.split(pat, line, maxsplit=1)
                parts = [p.strip() for p in parts]

                info['package'] = parts[0]
                if len(parts) > 1:
                    op, rest = parts[1:]
                    if ';' in rest:
                        # Handle platform specific dependencies
                        # http://setuptools.readthedocs.io/en/latest/setuptools.html#declaring-platform-specific-dependencies
                        version, platform_deps = map(str.strip, rest.split(';'))
                        info['platform_deps'] = platform_deps
                    else:
                        version = rest  # NOQA
                    info['version'] = (op, version)
            yield info

    def parse_require_file(fpath):
        with open(fpath, 'r') as f:
            for line in f.readlines():
                line = line.strip()
                if line and not line.startswith('#'):
                    for info in parse_line(line):
                        yield info

    def gen_packages_items():
        if exists(require_fpath):
            for info in parse_require_file(require_fpath):
                parts = [info['package']]
                if with_version and 'version' in info:
                    parts.extend(info['version'])
                if not sys.version.startswith('3.4'):
                    # apparently package_deps are broken in 3.4
                    platform_deps = info.get('platform_deps')
                    if platform_deps is not None:
                        parts.append(';' + platform_deps)
                item = ''.join(parts)
                yield item

    packages = list(gen_packages_items())
    return packages


INSTALL_REQUIRES = [
    #'PyQt 4/5'  # non pipi index
    'cachetools>=2.0.1',
]

NAME = 'guitool_ibeis'
MB_PYTHON_TAG = native_mb_python_tag()  # NOQA
VERSION = parse_version('guitool_ibeis/__init__.py')

# try:
#     from Cython.Distutils import build_ext
#     cmdclass = {'build_ext': build_ext}
# except Exception as ex:
#     print(ex)
#     print('WARNING: Cython is not installed. This is only a problem if you are building C extensions')
#     cmdclass = {}


try:
    class EmptyListWithLength(list):
        def __len__(self):
            return 1
except Exception:
    raise RuntimeError('FAILED TO ADD BUILD CONSTRUCTS')

if __name__ == '__main__':
    KWARGS = dict(
        name='guitool_ibeis',
        description=('Guitool - tools pyqt4 guis'),
        author='Jon Crall',
        long_description=parse_description(),
        long_description_content_type='text/x-rst',
        author_email='erotemic@gmail.com',
        packages=['guitool_ibeis', 'guitool_ibeis.__PYQT__'],
        url='https://github.com/Erotemic/guitool',
        # ext_modules=ext_modules,
        version=VERSION,
        # cmdclass=cmdclass,
        # keywords='',
        # package_data={},
        # classifiers=[],
        #language='c',
        install_requires=parse_requirements('requirements/runtime.txt'),
        extras_require={
            'all': parse_requirements('requirements.txt'),
            'tests': parse_requirements('requirements/tests.txt'),
            'build': parse_requirements('requirements/build.txt'),
            'runtime': parse_requirements('requirements/runtime.txt'),
        },
        license='Apache 2',
        # List of classifiers available at:
        # https://pypi.python.org/pypi?%3Aaction=list_classifiers
        classifiers=[
            'Development Status :: 4 - Beta',
            'License :: OSI Approved :: Apache Software License',  # Interpret as Apache License v2.0
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Utilities',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
        ],
    )
    import sysconfig
    import os
    try:
        soconfig = sysconfig.get_config_var('EXT_SUFFIX')
    except Exception:
        soconfig = sysconfig.get_config_var('SO')

    def get_lib_ext():
        if sys.platform.startswith('win32'):
            ext = '.dll'
        elif sys.platform.startswith('darwin'):
            ext = '.dylib'
        elif sys.platform.startswith('linux'):
            ext = '.so'
        else:
            raise Exception('Unknown operating system: %s' % sys.platform)
        return ext

    libext = get_lib_ext()
    _pyver = '{}.{}'.format(sys.version_info.major, sys.version_info.minor)
    hack_libconfig = '-{}{}'.format(_pyver, libext)

    PACKAGE_DATA = (
            ['*%s' % soconfig] +
            ['*%s' % hack_libconfig] +
            ['*%s' % libext] +
            (['*.dll'] if os.name == 'nt' else []) +
            (['Release\\*.dll'] if os.name == 'nt' else [])
            # ["LICENSE.txt", "LICENSE-3RD-PARTY.txt", "LICENSE.SIFT"]
    )
    if '--universal' in sys.argv:
        from setuptools import setup
    else:
        from skbuild import setup
        KWARGS.update(dict(
            ext_modules=EmptyListWithLength(),  # hack for including ctypes bins
            include_package_data=True,
            package_data={
                KWARGS['name']: PACKAGE_DATA,
            },
        ))
    setup(**KWARGS)
